/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "Map.h"
#include "Vector.h"
#include "SrcInfo.h"

using namespace std;
using namespace FD;

class CvfdhMatchShapes;
   
DECLARE_NODE(CvfdhMatchShapes)
/*Node
 *
 * @name CvfdhMatchShapes
 * @category FDOPENCV_HARK:Feature
 * @description CvfdhMatchShapes
 *
 * @input_name IMAGEIN
 * @input_type any
 * @input_description Input image. The type should be CvfdImage or Map<int, ObjectRef>.
 *
 * @input_name FIXIMAGEIN
 * @input_description Fixed base input image for matching
 * @input_type CvfdImage
 *
 * @input_name SRCIN
 * @input_type Vector<ObjectRef>
 * @input_description Source location. A type of an element of the vector is SrcInfo.
 *
 * @output_name FEATOUT
 * @output_type Map<int,ObjectRef>
 * @output_description Output features. The ObjectRef should be Vector<float>.
 *
 * @parameter_name CRITERIA
 * @parameter_type string
 * @parameter_value MATCH_I1
 * @parameter_list MATCH_I1:MATCH_I2:MATCH_I3
 * @parameter_description 
 *
 END*/
   
   
class CvfdhMatchShapes : public BufferedNode {

  int m_imageInID;
  int m_fiximageInID;
  int m_srcInID;
  int m_featOutID;

  string criteria;
      
public:
  CvfdhMatchShapes(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params), m_srcInID(-1)
  {
    m_imageInID = addInput("IMAGEIN");
    m_fiximageInID = addInput("FIXIMAGEIN");
    m_featOutID = addOutput("FEATOUT");
    criteria = dereference_cast<String>(parameters.get("CRITERIA")) ;
    inOrder = true;
  }      

  virtual int translateInput(string inputName) {
    if (inputName == "SRCIN") {
      m_srcInID = addInput(inputName);
      return m_srcInID;
    }
    else if (inputName == "IMAGEIN") {
      for (unsigned int i = 0; i < inputs.size(); i++) {
	if (inputs[i].name == inputName) {
	  return i;
	}
      }
    }
    else if (inputName == "FIXIMAGEIN") {
      for (unsigned int i = 0; i < inputs.size(); i++) {
	if (inputs[i].name == inputName) {
	  return i;
	}
      }
    }
    else {throw new NodeException(this, inputName+ " is not supported.", __FILE__, __LINE__);}
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {

    RCPtr<Map<int, ObjectRef> > outputMapPtr(new Map<int, ObjectRef>);
    (*(outputs[m_featOutID].buffer))[count] = outputMapPtr;

    RCPtr<Map<int, ObjectRef> > image_map(new Map<int, ObjectRef>);

    // Making the Map<id, IplImage*> for processing
    IplImage* img_in_tmp;
    ObjectRef in = getInput(m_imageInID, count);
    if (typeid(*in) == typeid(CvfdImage)) {
      RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
      if(imagePtr->getImage() == NULL){
	return;
      }
      img_in_tmp = cvCloneImage(imagePtr->getImage());
      CvfdImage* image= new CvfdImage(img_in_tmp);  
      (*image_map)[0] = image;
    }else if (typeid(*in) == typeid(Map<int, ObjectRef>)) {
      RCPtr<Map<int, ObjectRef> > imageMapPtr = getInput(m_imageInID, count);
      Map<int, ObjectRef>::iterator it = imageMapPtr->begin();
      for (; it != imageMapPtr->end(); ++it) {
        CvfdImage& imagePtr = object_cast<CvfdImage>(it->second);
	int store_flag = 1;
	if(m_srcInID != -1){
	  RCPtr<Vector<ObjectRef> > sources_ptr = getInput(m_srcInID, count);
	  const Vector<ObjectRef>& sources = *sources_ptr;
	  store_flag = 0;
	  for (int srci = 0; srci < sources.size(); srci++) {
	    RCPtr<SrcInfo> src = sources[srci];
	    if(src->id == it->first) store_flag = 1;
	  }
	}
	if(store_flag == 0) continue;
	img_in_tmp = cvCloneImage(imagePtr.getImage());
	CvfdImage* image= new CvfdImage(img_in_tmp);  
	(*image_map)[it->first] = image;
      }
    }

    RCPtr<CvfdImage> imagePtr2 = getInput(m_fiximageInID,count);   
    if(imagePtr2->getImage() == NULL){
      return;
    }

    IplImage* src_img2;
    src_img2 = cvCloneImage(imagePtr2->getImage());

    // Main routine
    Map<int, ObjectRef>::iterator img_it = image_map->begin();
    for (; img_it != image_map->end(); ++img_it) {
      IplImage* src_img1;      
      CvfdImage& imagePtr = object_cast<CvfdImage>(img_it->second);
      src_img1 = cvCloneImage(imagePtr.getImage());
    
      RCPtr<Vector<float> > map_contents(new Vector<float>);
    
      // ---------------------------------------------------------------
      // main routine : start

      double result;

      // char text[16];
      // int i;
      // CvFont font;
      // IplImage *dst_img;
      // dst_img = cvCloneImage (src_img2);  

      if(criteria == "MATCH_I1")
	result = cvMatchShapes (src_img1, src_img2, CV_CONTOURS_MATCH_I1, 0);
      if(criteria == "MATCH_I2")
	result = cvMatchShapes (src_img1, src_img2, CV_CONTOURS_MATCH_I2, 0);
      if(criteria == "MATCH_I3")
	result = cvMatchShapes (src_img1, src_img2, CV_CONTOURS_MATCH_I3, 0);

      // snprintf (text, 16, "%.5f", result);
      // cvInitFont (&font, CV_FONT_HERSHEY_SIMPLEX, 1.0, 1.0, 0, 4, 8);
      // cvPutText (dst_img, text, cvPoint (10, dst_img->height - 10), &font, cvScalarAll (0));

      (*map_contents).push_back((float)result);
      (*outputMapPtr)[img_it->first] = map_contents;    


      // IplImage* img_out;
      // img_out = cvCloneImage(dst_img);

      // main routine : finish
      // ---------------------------------------------------------------
         
      // CvfdImage* image= new CvfdImage(img_out);       
      // (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(image); 

      cvReleaseImage(&src_img1);
      // cvReleaseImage(&dst_img);
      // cvReleaseImage(&img_out);
    }

    cvReleaseImage(&img_in_tmp);
    cvReleaseImage(&src_img2);

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhMatchShapes)
};
