/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;

class CvfdAvg;
   
DECLARE_NODE(CvfdAvg)
/*Node
 *
 * @name CvfdAvg
 * @category FDOPENCV_ORG:Arithmetic Logic Comparison
 * @description Calculates average (mean) of image.
 *
 * @input_name IMAGE
 * @input_description Input image.
 * @input_type CvfdImage
 *
 * @output_name AVG_1
 * @output_description Average (mean) of the channel 1 of the image.
 * @output_type float
 *
 * @output_name AVG_2
 * @output_description Average (mean) of the channel 2 of the image.
 * @output_type float
 *
 * @output_name AVG_3
 * @output_description Average (mean) of the channel 3 of the image.
 * @output_type float
 *
 END*/
   
   
class CvfdAvg : public BufferedNode 
{
  //Input ID            
  int m_imageID;
  //Output ID
  int m_avg1ID;
  int m_avg2ID;
  int m_avg3ID;
      
public:
  CvfdAvg(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_imageID = addInput("IMAGE");
    //add outputs
    m_avg1ID = addOutput("AVG_1");
    m_avg2ID = addOutput("AVG_2");
    m_avg3ID = addOutput("AVG_3");
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    //Read the inputs
    RCPtr<CvfdImage> imagePtr = getInput(m_imageID,count);   
    //Handle
         
    CvScalar average = cvScalar(0,0,0,0);
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__;
    OPENCV_CALL(average = cvAvg( imagePtr->getImage() ));       
    __END__;
    cvSetErrMode( status );
         
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to mean the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      }
         
    (*(outputs[m_avg1ID].buffer))[count] = ObjectRef(Float::alloc( average.val[0]));
    (*(outputs[m_avg2ID].buffer))[count] = ObjectRef(Float::alloc( average.val[1]));
    (*(outputs[m_avg3ID].buffer))[count] = ObjectRef(Float::alloc( average.val[2]));
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdAvg)
};
